#!/bin/sh

# Copyright (c) 2013 SAS Institute Inc.
# Unpublished - All Rights Reserved
#
# emi_include.sh
# This file contains common variable and function definitions for use by
# SAS Environment Manager plugin scripts written by the EMI team.

# USAGE
# This script is intended to be sourced into other scripts and not run by itself
# Several variables are defined, but if already defined the existing value is used.
# This allows default behavior with an override mechanism

# The following variables are defined and can be overridden (* indicates the default)
#    EMI_LOG_DIR=[directory containing log files]
#    EMI_LOG_FILENAME=[relative path/file name]
#    EMI_LOG_LEVEL=[TRACE|DEBUG|*INFO|WARN|ERROR|OFF]
#    EMI_LOG_FILENAME_USE_TIMESTAMP=[true|false*]

# The following variables are defined by sourcing this script
# ---------------------------------------
# General Variables
# ---------------------------------------
# _SCRIPT_FILENAME - Identifies the absolute path to the calling script that sourced this file
# _SCRIPT_PATH     - Identifies the absolute path to the parent directory of the script
#                    that sourced this file
# _TIME_STR        - Identifies the time of execution of the script to the second
#                  - (e.g. 20130318_134510 = 3/18/13 1:45:10pm)
#

# ====================================================================
# Revision history
# ====================================================================
# Date        Developer   Change
# ----        ---------   ------
# 2013-03-15  brelli      Initial creation
# 2013-03-18  brelli      Automatically set _SCRIPT_FILENAME, handle non-agent execution
# 2013-03-19  brelli      New functions - LogSysError, LogTermWarn, LogTermError, fix bugs
# 2017-01-11  rnb         Modified for Viya

# ====================================================================
# VARIABLES
# ====================================================================
_INCLUDE_NAME="SAS EMI Team Common Functions"
_INCLUDE_VERSION="00.02.00"
_INCLUDE_BUILD_ID="201600000000"

# Get a string of yyyymmddhhmmss to use for identifying time of execution
_TIME_STR=$(date '+%Y%m%d_%H%M%S')
_TIME_8601=$(date "+%Y-%m-%dT%H:%M:%S.%6N%:z")
_term_int="0"

# Try to get the calling script filename
if [ "" == "$(expr match '$0' '.*\(emi_include.sh\)')" ] ; then
	if [ -f "$0" ] ; then
		# Determine name of script sourcing this file
                _SCRIPT_NAME=$(basename $0)
		_SCRIPT_FILENAME=$0
		_SCRIPT_PATH=$(dirname ${_SCRIPT_FILENAME})
		# Determine script file name and location (regardless of working directory)
		_INCLUDE_PATH=$(dirname ${BASH_SOURCE})
	fi
fi

# Configure logging
_LOG_LEVEL_TRACE="5"
_LOG_LEVEL_DEBUG="4"
_LOG_LEVEL_INFO="3"
_LOG_LEVEL_WARN="2"
_LOG_LEVEL_ERROR="1"
_LOG_LEVEL_OFF="0"

if [ "" == "${EMI_LOG_LEVEL}" ] ; then
    _LOG_LEVEL="INFO"
else
    _LOG_LEVEL="$(echo ${EMI_LOG_LEVEL} | tr '[:lower:]' '[:upper:]')"
fi

case ${_LOG_LEVEL} in
    "TRACE")
        _LOG_LEVEL_INT="${_LOG_LEVEL_TRACE}"
        ;;
    "DEBUG")
        _LOG_LEVEL_INT="${_LOG_LEVEL_DEBUG}"
        ;;
    "INFO")
        _LOG_LEVEL_INT="${_LOG_LEVEL_INFO}"
        ;;
    "WARN")
        _LOG_LEVEL_INT="${_LOG_LEVEL_WARN}"
        ;;
    "ERROR")
        _LOG_LEVEL_INT="${_LOG_LEVEL_ERROR}"
        ;;
    "OFF")
        _LOG_LEVEL_INT="${_LOG_LEVEL_OFF}"
        ;;
    *)
        # Setting _LOG_LEVEL to an invalid value effectively disables logging
        _LOG_LEVEL_INT="${_LOG_LEVEL_OFF}"
        ;;
esac

# may need to add emi log directory
if [ "" == "${EMI_LOG_DIR}" ] ; then
    # test for deployid, default location is /opt/sas/viya/config/var/log
    _LOG_DIR="$ROOTDIR/$SASDEPLOYID/config/var/log/evmcltsvcs/$SASINSTID"
else
    _LOG_DIR="${EMI_LOG_DIR}"
fi

if ! [ -d "${_LOG_DIR}" ] ; then
    mkdir -p ${_LOG_DIR} 2>/dev/null
    rc=$?
    if [ "$rc" != "0" ]; then
        echo "Failed to create [${_LOG_DIR}], using ."
        _LOG_DIR="."
    fi
fi

if [ "" != "$EMI_LOG_FILENAME" ] ; then
    _LOG_FILENAME="${EMI_LOG_FILENAME}"
else
    if [ "" == "${_LOG_FILENAME}" ] ; then
        if [ "" != "${_SCRIPT_FILENAME}" ] ; then
            if [ "true" == "${_LOG_FILENAME_USE_TIMESTAMP}" ] ; then
	           _LOG_FILENAME=$(basename $(echo ${_SCRIPT_FILENAME}_${_TIME_STR}.log))
            else
                _LOG_FILENAME=$(basename $(echo ${_SCRIPT_FILENAME}.log))
            fi
	   else
	       # Could not determine parent script filename
	       # _LOG_FILENAME and _LOG_DIR must have been set to enable logging in this case
	       _LOG_FILENAME=""
            _LOG_LEVEL="${_LOG_LEVEL_OFF}"
	   fi
    fi
fi

# ====================================================================
# FUNCTIONS
# ====================================================================

# Log()
#     Appends the provided message to the end of the configured log file.
#     This function ignores the configured _LOG_LEVEL and always logs the message.
# Pre-Condition:
#     _LOG_FILENAME has been set and the file is writable
# Parameters:
#     $1 - The message to append to the log file
Log() {
	if [ ! -d "${_LOG_DIR}" ] && [ ! -f "${_LOG_DIR}" ]; then
		mkdir -p "${_LOG_DIR}"
	fi

    if [ -f "${_LOG_DIR}/${_LOG_FILENAME}" ] ; then
        if [ -w "${_LOG_DIR}/${_LOG_FILENAME}" ] ; then
            _LOG_WRITABLE="true"
        fi
    else
        if [ -w "${_LOG_DIR}" ] ; then
            _LOG_WRITABLE="true"
        fi
    fi

    if [ "true" == "${_LOG_WRITABLE}" ] && [ "" != "${_LOG_FILENAME}" ] ; then
        echo "$_TIME_8601 $@" >> "${_LOG_DIR}/${_LOG_FILENAME}"
    else
        if [ "true" != "${_WARNED_UNABLE_TO_WRITE_LOG}" ] ; then
            echo "WARNING: Unable to write log file [${_LOG_DIR}/${_LOG_FILENAME}]"
            _WARNED_UNABLE_TO_WRITE_LOG=true
        fi
    fi
}

# LogTerm()
#     Log to terminal and an info message to the log file.  Simply use 'echo' to avoid
# Parameters:
#     $1 - The message
LogTerm() {
    _term_int="3"
    if [ "${_LOG_LEVEL_INT}" -ge "${_term_int}" ] ; then
        Log "INFO [] [$_SCRIPT_NAME] - $@"
        echo "$@"
    fi
}

# LogTermDebug()
#     Log to terminal and a debug message to the log file.  Simply use 'echo' to avoid
#     logging to the log file. The message is only logged if the _LOG_LEVEL is set to
#     _LOG_LEVEL_DEBUG or higher
# Parameters:
#     $1 - The message
LogTermDebug() {
    _term_int="4"
    if [ "${_LOG_LEVEL_INT}" -ge "${_term_int}" ] ; then
        echo "$@"
    fi
    LogDebug "$@"
}

# LogSysError()
#     Log to error stream and an error message to the log file.  Simply use 'echo' to avoid
#     logging to the log file. The message is only logged if the _LOG_LEVEL is set to
#     _LOG_LEVEL_ERROR or higher
# Parameters:
#     $1 - The message
LogSysError() {
    echo "$@" >&2
    LogError "$@"
}

# LogTermInfo()
#     Log to terminal and an info message to the log file.  Simply use 'echo' to avoid
#     logging to the log file. The message is only logged if the _LOG_LEVEL is set to
#     _LOG_LEVEL_INFO or higher
# Parameters:
#     $1 - The message
LogTermInfo() {
    _term_int="3"
    if [ "${_LOG_LEVEL_INT}" -ge "${_term_int}" ] ; then
        echo "$@"
    fi
    LogInfo "$@"
}

# LogTermWarn()
#     Log to terminal and a warning message to the log file.  Simply use 'echo' to avoid
#     logging to the log file. The message is only logged if the _LOG_LEVEL is set to
#     _LOG_LEVEL_WARN or higher
# Parameters:
#     $1 - The message
LogTermWarn() {
    _term_int="2"
    if [ "${_LOG_LEVEL_INT}" -ge "${_term_int}" ] ; then
        echo "$@"
    fi
    LogWarn "$@"
}

# LogTermError()
#     Log to terminal and an error message to the log file.  Simply use 'echo' to avoid
#     logging to the log file. The message is only logged if the _LOG_LEVEL is set to
#     _LOG_LEVEL_ERROR or higher
# Parameters:
#     $1 - The message to append to the log file
LogTermError() {
    _term_int="1"
    if [ "${_LOG_LEVEL_INT}" -ge "${_term_int}" ] ; then
        echo "$@"
    fi
    LogError "$@"
}

# LogDebug()
#     Log debug message (usually not enabled by default)
#     The message is only logged if the _LOG_LEVEL is set to _LOG_LEVEL_DEBUG or higher
# Parameters:
#     $1 - The message to append to the log file
LogDebug() {
    if [ "${_LOG_LEVEL_INT}" -ge "${_LOG_LEVEL_DEBUG}" ] ; then
        Log "DEBUG [] [$_SCRIPT_NAME] - $@"
    fi
}

# LogInfo()
#     Log info message (usually enabled by default)
#     The message is only logged if the _LOG_LEVEL is set to _LOG_LEVEL_INFO or higher
# Parameters:
#     $1 - The message to append to the log file
LogInfo() {
    if [ "${_LOG_LEVEL_INT}" -ge "${_LOG_LEVEL_INFO}" ] ; then
        Log "INFO [] [$_SCRIPT_NAME] - $@"
    fi
}

# LogWarn()
#     Log warning message
#     The message is only logged if the _LOG_LEVEL is set to _LOG_LEVEL_WARN or higher
# Parameters:
#     $1 - The message to append to the log file
LogWarn() {
    if [ "${_LOG_LEVEL_INT}" -ge "${_LOG_LEVEL_WARN}" ] ; then
        Log "WARN [] [$_SCRIPT_NAME] - $@"
    fi
}

# LogError()
#     Log error message
#     The message is only logged if the _LOG_LEVEL is set to _LOG_LEVEL_ERROR or higher
# Parameters:
#     $1 - The message to append to the log file
LogError() {
    if [ "${_LOG_LEVEL_INT}" -ge "${_LOG_LEVEL_ERROR}" ] ; then
        Log "ERROR [] [$SCRIPT_NAME] - $@"
    fi
}
