#!/bin/bash

# Copyright (c) 2017 SAS Institute Inc. 
# Unpublished - All Rights Reserved 
#
# emi-sysinfo.sh
# script to collect and report system attributes

#
# USAGE
# Usage: emi-sysinfo [ -h ] [ -v ] [ --warningRC rc ] 
#

# ====================================================================
# Revision history
# ====================================================================
# Date        Developer   Change
# ----        ---------   ------
# 2017-02-17  bnb         Coded

EMI_HOSTNAME=$(hostname -s)		    # The hostname of this server
TMPFILE="/tmp/$(basename $0).$$"	# Temporary File

SCRIPT_NAME=$(basename $0)
SCRIPT_VERSION="Viya.201702152017"
SCRIPT_BUILD_ID="201702152017"
SCRIPT_PATH=$(dirname $0)
SCRIPT_CANONICAL_PATH=$(dirname $(readlink -f $0))

if [ ! $EMIROOT ] ; then 
[[ ${SCRIPT_CANONICAL_PATH} =~ ^(.*)/([^/]+)/home/.*$ ]] && EMIROOT=${BASH_REMATCH[1]}
fi

ROOTDIR=${EMIROOT:-/opt/sas}
SASDEPLOYID=${SASDEPLOYID:-viya}
SASINSTID=${SASINSTID:-default}
SASCONFIG=$ROOTDIR/$SASDEPLOYID/config
SASHOME=$ROOTDIR/$SASDEPLOYID/home

RC_SUCCESS=0
RC_WARN=1
RC_ERROR=2
exitrc=$RC_SUCCESS

PATH=/sbin:/usr/sbin:/bin:/usr/bin:${SCRIPT_PATH}
_LOG_FILENAME="$(basename $0).log"

# Include common functions
. "${SASHOME}/bin/emi-include.sh"

# ====================================================================
# FUNCTIONS
# ====================================================================
emi_error_trap_handler()
{
    LASTLINE="$1"            # argument 1: last line of error occurence
    LASTERR="$2"             # argument 2: error code of last command
    LogTermError "line [${LASTLINE}]: exit status of last command: [${LASTERR}]"
    exitrc=$RC_WARN
}

ShowUsage() {
    LogTerm "Usage: $_SCRIPT_FILENAME [ -h ] [ -d ] [ --warningRC rc ] "
}

emi_get_system() {
	local host=$1
	system=("hosts/$host/system/kernel-name=$(uname -s 2>/dev/null)"$'\n'
	   		"hosts/$host/system/kernel-release=$(uname -r 2>/dev/null)"$'\n'
	   		"hosts/$host/system/kernel-version=$(uname -v 2>/dev/null)"$'\n'
	    	"hosts/$host/system/processor=$(uname -p 2>/dev/null)"$'\n'
	    	"hosts/$host/system/machine=$(uname -m 2>/dev/null)"$'\n'
	    	"hosts/$host/system/operating-system=$(uname -o 2>/dev/null)"$'\n'
	    	"hosts/$host/system/hardware-platform=$(uname -i 2>/dev/null)"$'\n'
	    	"hosts/$host/system/memory-totalMB=$(free -m | grep -oP '\d+'|sed -n '1p')"$'\n'
			"hosts/$host/system/memory-sharedMB=$(free -m | grep -oP '\d+'|sed -n '4p')"$'\n'
			"hosts/$host/system/memory-buffersMB=$(free -m | grep -oP '\d+'|sed -n '5p')"$'\n'
			"hosts/$host/system/memory-cachedMB=$(free -m | grep -oP '\d+'|sed -n '6p')"$'\n'
			"hosts/$host/system/distro=$(head -n 1 /etc/issue 2>/dev/null)"$'\n'
			"hosts/$host/system/timezone=$(date +%Z)"$'\n'
			"hosts/$host/system/tzoffset=$(date +%:z)"$'\n'
#			"hosts/$host/system/uptime=$(uptime | sed 's/^.* up \+\(.\+\), \+[0-9] user.*$/\1/' 2>/dev/null)"$'\n'			
	    	)
}

emi_get_license() {
	local host=$1
	license=("hosts/$host/license/site=$(cat /opt/sas/viya/home/SASFoundation/SASViya*.txt|sed -n 's/.*SITE=\([^ ]*\).*/\1/p' 2>/dev/null)"$'\n'
			"hosts/$host/license/license=$(cat /opt/sas/viya/home/SASFoundation/SASViya*.txt|sed -n 's/.*LICENSE=\(SAS [^ ;]*\).*/\1/p' 2>/dev/null)"$'\n'
			)
}

emi_get_naming() {
	local host=$1
	knownby=("hosts/$host/knownas/hostname-short=$(hostname -s)"$'\n'
			"hosts/$host/knownas/hostname-fqdn=$(hostname -f)"$'\n'
			"hosts/$host/knownas/dns-domain-name=$(hostname -d)"$'\n'
			)
}

emi_get_session() {
	local host=$1
	session=("hosts/$host/ulimits/user=$(whoami )"$'\n'
			"hosts/$host/ulimits/maxuserprocesses=$(ulimit -u )"$'\n'
			"hosts/$host/ulimits/cputime=$(ulimit -t )"$'\n'
			"hosts/$host/ulimits/corefilesize=$(ulimit -c )"$'\n'
			"hosts/$host/ulimits/datasegsize=$(ulimit -d )"$'\n'
			"hosts/$host/ulimits/filesize=$(ulimit -f )"$'\n'
			"hosts/$host/ulimits/filelocks=$(ulimit -x )"$'\n'	
			"hosts/$host/ulimits/openfiles=$(ulimit -n )"$'\n'
			"hosts/$host/ulimits/maxlockmemory=$(ulimit -l)"$'\n'							
			"hosts/$host/ulimits/maxmemorysize=$(ulimit -m )"$'\n'
			"hosts/$host/ulimits/pipesize=$(ulimit -p )"$'\n'
			"hosts/$host/ulimits/virtualmemory=$(ulimit -v )"$'\n'
			)
}

emi_get_sasrpms() {
	local host=$1
	rpms=()
	for package in $(rpm -qa|grep sas-); do
   		pname=${package%%.*}
   		rpms+="hosts/$host/rpms/${pname%-*}=$package"$'\n'
	done
}

# trap commands with non-zero exit code
#
trap 'emi_error_trap_handler ${LINENO} $?' ERR

LogInfo "Executing"
LogTermDebug "EMI_LOG_LEVEL = [${EMI_LOG_LEVEL}]"
# ====================================================================
# Pulling off options, prepping for execution of requested function 
# ====================================================================
while : 
do
	LogDebug "CLI arg looping $1"
	case "$1" in
    	-h | --help)
    		ShowUsage
			exit $RC_SUCCESS
			;;
		-d | --debug)
			_LOG_LEVEL_INT="4"
      		shift
      		;;
    	--warningRC)
    		RC_WARN="$2"
			shift 2
      		;;     
    	-*) 
      		LogTerm "Error: Unknown option: $1"
      		exit $RC_ERROR
      		;;
    	*)  # No more options
         	break
      		;;
  	esac
done

LogTermDebug "PATH set to [$PATH]"
LogDebug "Log Directory    = ${_LOG_DIR}"
LogDebug "Log Filename     = ${_LOG_FILENAME}"

# test for existence of emi-config. If missing, exit.
KVCMD=$(which emi-config )
LogTermDebug "which emi-config [$KVCMD]" 

if ! [ -f "$KVCMD" ]; then
	LogTermError "Cannot find emi-config, PATH=[$PATH]"
	exit $RC_ERROR
fi

# test for existence of token file to read/write to consul
if ! [ -f "$CONSUL_TOKEN" ]; then
	TOKEN=/opt/sas/viya/config/etc/SASSecurityCertificateFramework/tokens/consul/default/client.token
	if ! [ -f "$TOKEN" ]; then
		LogTermError "Cannot find [$TOKEN] command"
		ShowUsage 
		exit $RC_ERROR
	fi
fi

#
# get system stuff
# 
emi_get_system $EMI_HOSTNAME
if [ ${_LOG_LEVEL_INT} -ge 4 ]; then
	echo "${system[*]}" 
fi

#
# Load system values
#
LogDebug "Loading system info"
$KVCMD -base ops load -values "${system[*]}"
rc=$?
if [ "$rc" != "0" ]; then
	LogTermError "emi-config system load failed rc=[$rc]"
	exitrc=$RC_WARN
fi

#
# get naming info
#
emi_get_naming $EMI_HOSTNAME
if [ ${_LOG_LEVEL_INT} -ge 4 ]; then
	echo "${knownby[*]}"
fi

#
# Load naming info
#
LogDebug "Loading knownby info"
$KVCMD -base ops load -values "${knownby[*]}"
rc=$?
if [ "$rc" != "0" ]; then
	LogTermError "emi-config knownby load failed rc=[$rc]"
	exitrc=$RC_WARN
fi

#
# get session (ulimit) for running user
#
emi_get_session $EMI_HOSTNAME
if [ ${_LOG_LEVEL_INT} -ge 4 ]; then
	echo "${session[*]}" 
fi

#
# Load session info
#
LogDebug "Loading session (ulimits)"
$KVCMD -base ops load -values "${session[*]}"
rc=$?
if [ "$rc" != "0" ]; then
	LogTermError "emi-config session (ulimits) load failed rc=[$rc]"
	exitrc=$RC_WARN
fi

#
# get license info
#
emi_get_license $EMI_HOSTNAME
if [ ${_LOG_LEVEL_INT} -ge 4 ]; then
	echo "${license[*]}" 
fi

LogDebug "Loading license info"
$KVCMD -base ops load -values "${license[*]}"
rc=$?
if [ "$rc" != "0" ]; then
	LogTermError "emi-config license load failed rc=[$rc]"
	exitrc=$RC_WARN
fi

#
# get sas rpm list 
#
emi_get_sasrpms $EMI_HOSTNAME
if [ ${_LOG_LEVEL_INT} -ge 4 ]; then
	echo "${rpms[*]}" 
fi

#
# Load rpm list
#
LogDebug "Loading rpms"
$KVCMD -base ops load -values "${rpms[*]}"
rc=$?
if [ "$rc" != "0" ]; then
	LogTermError "emi-config rpms load failed rc=[$rc]"
	exitrc=$RC_WARN
fi

#
# Load network addresses
#
i=0
for network in $(hostname -I|sed -n '1p'); do
	let i=i+1
	LogDebug "Network $i $network"
	$KVCMD -base ops load -values "hosts/$EMI_HOSTNAME/knownas/addr$i=$network"
done
#
# add lastupdate record
# 
$KVCMD -base ops load -values "hosts/$EMI_HOSTNAME/lastupdate=$(date +%F:%T)"

exit $exitrc
