#!/bin/bash

# If no application is given the default is that all types of audit records are extracted.
# The user can only give one application type per run.  The application is part of the filter.
# See filter below.
if [ -z "$1" ]
then
   echo No arguments given.  Defaults for all arguments will be used.
else
   usage="$1"
   if [ "$usage" == "help" ]
   then
   {
   echo ""
   echo "Usage: ./genAudit.sh"
   echo "       ./genAudit.sh -r reports,folders -l 1000 -d 7"
   echo ""
   exit 0
   }
   fi
fi

verbose=""

while getopts ":a:l:d:v" opt; do
   case $opt in
      a)
        set -f
        IFS=','
        array=($OPTARG) ;;
      l)
         limit=${OPTARG} ;;
      d)
         auditDays=${OPTARG} ;;
      v)
         verbose=--verbose ;;
   esac 

done

SCRIPT_PATH=$(dirname $0)
SCRIPT_CANONICAL_PATH=$(dirname $(readlink -f $0))

if [ ! $EMIROOT ] ; then
  [[ ${SCRIPT_CANONICAL_PATH} =~ ^(.*)/([^/]+)/home/.*$ ]] && EMIROOT=${BASH_REMATCH[1]}
fi

if [ ! $SASDEPLOYID ]; then
   SASDEPLOYID=${BASH_REMATCH[2]}
fi

ROOTDIR=${EMIROOT:-/opt/sas}
SASCONFIG=$ROOTDIR/$SASDEPLOYID/config
SASHOME=$ROOTDIR/$SASDEPLOYID/home

PATH=/sbin:/usr/sbin:/bin:/usr/bin:${SCRIPT_PATH}

# Include common functions
. "${SASHOME}/bin/emi-include.sh"

SAS_BIN_EXE=${SAS_BIN_EXE:-$SASHOME/SASFoundation/sas}

# find the cashed location.  if it doesn't exist, make it.
cacheLocation=${SASCONFIG}/var/cache/auditcli
if [ ! -d $cacheLocation ]
then
   mkdir $cacheLocation
fi

# The append file is the file each audit extraction block is sent to.  Each extraction
# block is then appended to the audit.csv file.
appendFile="$cacheLocation/AuditAppendFile56791d72cc67.csv"
file="$cacheLocation/audit.csv"
	  
BIN=${SASHOME}/bin
  
export SAS_OUTPUT=text
export EMI_OAUTH_TOKEN=$($BIN/emi-util token -id sas.ops-agentsrv)
export SAS_SERVICES_ENDPOINT=$($BIN/emi-config http)
export SYSTEM_DATA=$(echo $($BIN/sas-admin -k --output text cas settings list | grep system.caslib) | cut -d " " -f2- | xargs)

# only run if the current HOST is in the list of controllers.
CURRHOST=$(hostname --long)		    # The hostname of this server
hosts=$($BIN/sas-admin -k cas --output json servers list | grep $CURRHOST | cut -d\" -f4)
servers=$($BIN/sas-admin -k cas --output json servers list | grep name | cut -d\" -f4 | tr '\n' ' ')

for i in $hosts; do
   if [ $i == "$CURRHOST" ]
   then
      casServer=`echo $servers | sed 's/ .*//'`

      # Find the sysData path where the resulting audit file should go. 
      sysDataPath=$($BIN/sas-admin -k --output text cas caslibs show-info --server $casServer --caslib $SYSTEM_DATA | grep sysData)
      sysDataPath="${sysDataPath##* }"
      if [ ! -d $sysDataPath ]; then
         continue
      fi
   break
   fi

done

if [ ! -d $sysDataPath ]; then
   echo "This host is not the cas controller.  Exitting the genAudit script."
   exit 0
fi
   
# If no limit is given then the default limit is 5000 records per extraction.
# This limit represents the maximumn number of audit records per filter.
# See filter below.
if [ -z "$limit" ]
then
   echo "No block limit given.  Default block of records extracted is 1000."
   limit=1000
fi

# If no auditDays are given then the default number of days to go back in history is 7 days.
# If the the audit file does not exist then the timestamp for the filter will go back auditDays
# number of days.
if [ -z "$auditDays" ]
then
   echo "No maximum number of days of audit records given.  The default number of auditing days will be 7."
   auditDays=7
fi

# Initialize fromDate and generate toDate.  The toDate is the current time in UTC.
# Use Z instead of UTC as audit record timestamps use Z.  The filter needs to match
# the audit record timestamp for comparison purposes.
toDate=$(date --utc +%Y-%m-%dT%H:%M:%S.000%Z)
toDate="${toDate/UTC/Z}"

# Use today's current time minus auditDays to generate the from date used in the filter.
today=$(date) 
echo "Starting the audit extraction from " $auditDays " days ago."

# Generate the from date used in the filter.
initialFromDate=$(date --utc --date="${today} -${auditDays} day" +%Y-%m-%dT%H:%M:%S.000%Z)

# Quiet can be used to suppress .csv header.  This is needed if there are mulitple
# blocks of records retrieved.  The first block will need the header, subsequent blocks will not.
quiet=""

# if the audit.csv file already exists
# remove audit records auditDays+1 (ex: if auditDays is 7 then 8) days old
if [ -f "$file" ]
then
   oneDayEarlier=$(($auditDays+1))
   truncateDate=$(date --utc --date="${today} -${oneDayEarlier} day" +%Y-%m-%dT)
   grep -v "$truncateDate" $file > $file.tmp && mv $file.tmp $file
   quiet="--no-header"
fi

applicationFile=""

for i in ${array[@]}; do
   fromDate="${initialFromDate/UTC/Z}"
   resolveURI=""
   application="--application"
   applicationFile="$cacheLocation/audit_${i}.fromDate"
   if [ -f "$applicationFile" ]
   then
      fromDate=$(cat $applicationFile | xargs)
   fi

   if [ "${i}" == "reports" ]
   then
      resolveURI="--resolve-uri"
   elif [ "${i}" == "dataPlans" ]
   then
      resolveURI="--resolve-uri"
   fi

   # Even with the filter the number of records can be large.  Extract the records given the limit and
   # keep extracting until all the audit records for the given filter are retrieved.
   while :
   do
      echo "Extracting ${i} audit records from " $fromDate " to " $toDate
   
      # Use the sas-audit CLI to extract the audit records.
      $BIN/sas-admin -k $verbose audit list $quiet --limit $limit --sort-by "timeStamp" $application ${i} --csv $appendFile --after $fromDate --before $toDate $resolveURI
      isDone=$?

      if [ "$isDone" != "0" ]
      then
         echo There was a problem extracting ${i} audit records.
         if [ -f "$appendFile" ]
         then
            rm $appendFile
         fi
	  
	  break
      fi

      numberOfRecords=0
      if [ -f "$appendFile" ]
      then
         numberOfRecords=$(wc -l $appendFile|cut -d' ' -f 1)
         if [ "$quiet" == "" ]
	     then
            numberOfRecords=$(($numberOfRecords - 1))
	     fi
      fi
      if [ "$numberOfRecords" == "0" ]
      then
         
         if [ -f "$appendFile" ]
         then
            rm $appendFile
		 fi
	  
         break
   
      else
         # Concatenate each extrated append file to the audit file.
         # double check that there are records to concatenate.
         numberOfRecords=$(wc -l $appendFile|cut -d' ' -f 1)
         if [ "$numberOfRecords" == "0" ]
         then
		    # do not concatenate.  just break out of the loop and go to the next application.
            rm $appendFile
		    break
		 fi
         cat $appendFile >> $file

         # If there are more records to be extracted use the last line of audit append file to generate
         # a new from date.
         fromDate=$(echo $(tail -1 $appendFile)|cut -d' ' -f 2)
         echo $fromDate > $applicationFile
	     rm $appendFile

         # Subsequent extractions do not need a header for the .csv file.
         quiet="--no-header"
      fi

   done
   
done

# done
# take the resulting audit.csv and move it to cas.
# remove the audit table first.
# the first time, this might cause an error in the unload.
# ignore that error as it will only happen the first round.
if [ -f "$file" ]
then
   $BIN/sas-admin -k cas tables unload --server $casServer --caslib $SYSTEM_DATA --table AUDIT
   curl -k --include -HAuthorization:"bearer $EMI_OAUTH_TOKEN" -HAccept:application/json --location --max-time 30 -F tableName=AUDIT -F format=csv -F scope=global -F containsHeaderRow=true -F file=@$file --request POST $SAS_SERVICES_ENDPOINT/casManagement/servers/$casServer/caslibs/$SYSTEM_DATA/tables

fi

exit 0